/// @addtogroup covrel
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file graphhset3D.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _GraphicsHSet3DClass
#define _GraphicsHSet3DClass

#include "krak/krak.h"
#include "krak/coord3d.h"
#include "covrel/hset3D.h"

namespace capd{
namespace covrel{

class GraphicsHSet3D : public hSet3D{
public:
   typedef hSet3D::FloatVector FloatVector;
   typedef hSet3D::IntervalVector IntervalVector;
   typedef hSet3D::FloatMatrix FloatMatrix;
   typedef hSet3D::IntervalMatrix IntervalMatrix;

   GraphicsHSet3D(){}
   GraphicsHSet3D(const FloatVector &Center, const FloatVector &U, const FloatVector &S1, const FloatVector &S2)
      : hSet3D(Center,U,S1,S2){}
   GraphicsHSet3D(const IntervalVector &Center, const IntervalVector &U, const IntervalVector &S1, const IntervalVector &S2)
      : hSet3D(Center,U,S1,S2){}
   ~GraphicsHSet3D(){}

   void show(Frame &f, int dim1, int dim2) const;
   void show(Frame &f, int dim1, int dim2, const IntervalVector &, int color=BLACK) const;
   void show(Coord3D &f) const;
   void show(Coord3D &f, const IntervalVector &iv, int color=BLACK) const;
   friend Coord3D& operator << (Coord3D &, const GraphicsHSet3D&);
};

}} // namespace capd::covrel

#endif // _GraphicsHSet3DClass

/// @}
