/// @addtogroup diffIncl
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file DiffInclusionIntf.h
///
/// @author Tomasz Kapela
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/* Author: Tomasz Kapela, 2007 */

#ifndef _DIFF_INCLUSION_INTF_H_
#define _DIFF_INCLUSION_INTF_H_

#include <string>
#include <map>
#include "vectalg/normintf.h"
#include "dynsys/taylorintf.h"

namespace capd{
namespace diffIncl{

template<typename MapT>
class DiffInclusion : public capd::dynsys::Taylor<typename MapT::MapType>{

public:
  typedef MapT MultiMapType;
  typedef typename MapT::MapType MapType;
  typedef typename MapT::FunctionType FunctionType;
  typedef typename MapType::MatrixType MatrixType;
  typedef typename MatrixType::RowVectorType VectorType;
  typedef typename MatrixType::ScalarType ScalarType;
  typedef Norm<VectorType,MatrixType> NormType;

  DiffInclusion(   MultiMapType& diffIncl,               // map of the form f(x)+ g(x,e)
                   int Order,                            // order for integration
                   ScalarType const & Step,              // time step for integration
                   NormType const & norm                 // norm 
  );

  ~DiffInclusion();
  DiffInclusion & operator=(const DiffInclusion & a_t);
  
  VectorType diffInclusionEnclosure(const VectorType& x) const;
   
  using capd::dynsys::Taylor<MapType>::Phi;
  using capd::dynsys::Taylor<MapType>::JacPhi;
  using capd::dynsys::Taylor<MapType>::Remainder;
  using capd::dynsys::Taylor<MapType>::JacRemainder;
  using capd::dynsys::Taylor<MapType>::enclosure;
  using capd::dynsys::Taylor<MapType>::jacEnclosure;
  using capd::dynsys::Taylor<MapType>::getField;
  using capd::dynsys::Taylor<MapType>::getOrder;
  using capd::dynsys::Taylor<MapType>::getStep;
  using capd::dynsys::Taylor<MapType>::getCoeff;
  using capd::dynsys::Taylor<MapType>::getMatrixCoeff;
  using capd::dynsys::Taylor<MapType>::setStep;
  using capd::dynsys::Taylor<MapType>::setParameter;
  using capd::dynsys::Taylor<MapType>::setOrder;
  using capd::dynsys::Taylor<MapType>::setField;

protected:
  NormType * m_norm;
  MultiMapType & m_diffIncl;
  
  using capd::dynsys::Taylor<MapType>::computeMatrixCoeff;
  using capd::dynsys::Taylor<MapType>::m_vField;
  using capd::dynsys::Taylor<MapType>::m_order;
  using capd::dynsys::Taylor<MapType>::m_step;
  using capd::dynsys::Taylor<MapType>::m_coeff;
  using capd::dynsys::Taylor<MapType>::m_matrixCoeff;
  using capd::dynsys::Taylor<MapType>::m_dimension;
  using capd::dynsys::Taylor<MapType>::m_F;
};


// --------------- inline definitions -----------------

//###########################################################//


template<typename MapType>
inline DiffInclusion<MapType>& DiffInclusion<MapType>::operator=(const DiffInclusion& a_t)
{
  capd::dynsys::Taylor<MapType>::operator=(a_t);
  m_norm = (a_t.m_norm)->clone();
  m_diffIncl = a_t.m_diffIncl;
  return *this;
}

//###########################################################//


}} // namespace capd::diffIncl

#endif // _DIFF_INCLUSION_INTF_H_

/// @}
