/// @addtogroup diffIncl
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file inclInclRect2Set.hpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _InclInclRect2Set_HPP
#define _InclInclRect2Set_HPP

#include "diffIncl/inclRect2SetIntf.h"
#include <iostream>

namespace capd{
namespace diffIncl{


template<typename MatrixType>
void InclRect2Set<MatrixType>::move(capd::dynsys::DynSys<MatrixType>& dynsys)
{ 
  
  VectorType x = VectorType(*this);
  
  //computation of an unperturbed trajectory
  capd::dynset::Rect2Set<MatrixType>::move(dynsys);
  
  //computation of the influence of the perturbations
  VectorType Deltha = dynsys.perturbations(x); 
  
  // Rearrangements 
  x = midVector( m_x + Deltha);
  VectorType dr = m_x + Deltha - x;
  MatrixType BT = Transpose(m_B);
  
  m_r = m_r + BT * dr;
  m_x = x;

}

template<typename MatrixType>
void InclRect2Set<MatrixType>::move(capd::dynsys::DynSys<MatrixType>& dynsys, InclRect2Set<MatrixType>& result) const
{
    capd::dynset::Rect2Set<MatrixType>::move(dynsys, result);
    throw(" InclRect2Set::move(..., ...) not finished yet !!!");
}

template<typename T>
void corners(T& head, const T & tail, int i, int dim, std::vector<T> & cor)
{ 
  if(i < dim)
  { 
    head[i] = left(tail[i]);
    corners(head, tail, i+1, dim, cor);
    head[i] = right(tail[i]);
    corners(head, tail, i+1, dim, cor);
    head[i] = tail[i];
  }
  else
  {
    cor.push_back(head);
  }
}

template<typename MatrixType>
std::vector<typename InclRect2Set<MatrixType>::VectorType> InclRect2Set<MatrixType>::getCorners() const
{
  std::vector<VectorType> cor;
  VectorType v = m_r;
  corners(v, m_r, 0, v.dimension(), cor);
  for(typename std::vector<VectorType>::iterator it = cor.begin(); it != cor.end(); ++it){
    *it = m_x + m_C*m_r0 + m_B * *it; 
  }
  return cor;
}
}} // namespace capd::diffIncl

#endif // _InclRect2Set_HPP

/// @}
