/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file intvset.hpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _IntervalSet_HPP
#define _IntervalSet_HPP

#include "dynset/intvsetintf.h"

// move  by  Jac*x + remainder
namespace capd{
namespace dynset{

template<typename MatrixType>
typename IntervalSet<MatrixType>::ScalarType IntervalSet<MatrixType>::size(void)
{
   return ::size(m_x+m_r);
}

template<typename MatrixType>
void IntervalSet<MatrixType>::move(capd::dynsys::DynSys<MatrixType>& dynsys)
{
   VectorType s(m_x.dimension());

   VectorType xx = m_x+m_r;
   VectorType y = dynsys.Phi(m_x);
   split(y,s);
   s += dynsys.Remainder(xx);
   m_x = y;
   m_r = dynsys.JacPhi(xx)*m_r + s;
}

template<typename MatrixType>
void IntervalSet<MatrixType>::move(capd::dynsys::DynSys<MatrixType>& dynsys, IntervalSet& result) const
{
   VectorType s(m_x.dimension());

   VectorType xx = m_x + m_r;
   VectorType y = dynsys.Phi(m_x);
   split(y,s);
   s += dynsys.Remainder(xx);
   result.m_x = y;
   result.m_r = dynsys.JacPhi(xx)*m_r + s;
}

template<typename MatrixType>
std::string IntervalSet<MatrixType>::show(void) const
{
   std::ostringstream descriptor;
   descriptor << "IntervalSet: x=";
   descriptor << m_x << " r=";
   descriptor << m_r << " ";
   return descriptor.str();
}

template<typename MatrixType>
typename IntervalSet<MatrixType>::VectorType IntervalSet<MatrixType>::affineTransformation(
      const MatrixType& A_M, const VectorType& A_C) const
{
   return A_M*(m_x+m_r-A_C);
}

}} // namespace capd::dynset

#endif

/// @}
