/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file c2taylintf.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2003-2005 */

#ifndef _C2TaylIntf_H
#define _C2TaylIntf_H

#include <string>
#include <vector>
#include "vectalg/normintf.h"
#include "dynsys/taylorintf.h"
#include "dynsys/c2dynsys.h"
#include "dynsys/basicC2TaylorIntf.h"

namespace capd{
namespace dynsys{

template<typename MapT>
class C2Taylor : public Taylor<MapT>, public BasicC2Taylor<MapT>, public C2DynSys<typename MapT::MatrixType>{
public:
   typedef MapT MapType;
   typedef typename MapT::FunctionType FunctionType;
   typedef typename MapType::MatrixType MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef Norm<VectorType,MatrixType> NormType;
   typedef typename MapType::C2CoeffType C2CoeffType;

   C2Taylor(const MapType& vectorField, int Order, const ScalarType& Step);
   C2Taylor(const std::string& vectorField, int Order, const ScalarType& Step);
   C2Taylor(const char* vectorField, int Order, const ScalarType& Step);
   C2Taylor& operator=(const C2Taylor&);

   void c2Phi(const VectorType& x, MatrixType& jac, C2CoeffType& hessian) const;
   void c2Enclosure(
         const VectorType& W1,
         const MatrixType& W3,
         NormType& the_norm,
         C2CoeffType& result
   ) const;
   void c2Remainder(
         const VectorType& Enclosure,
         const MatrixType& jacEnclosure,
         const C2CoeffType& hessianEnclosure,
         VectorType& Remainder,
         MatrixType& jacRemainder,
         C2CoeffType& hessianRemainder // here the result is stored
   ) const;

   using BasicC2Taylor<MapType>::setField;
   using BasicC2Taylor<MapType>::setOrder;

   using Taylor<MapType>::Phi;
   using Taylor<MapType>::JacPhi;
   using Taylor<MapType>::Remainder;
   using Taylor<MapType>::JacRemainder;
   using Taylor<MapType>::enclosure;
   using Taylor<MapType>::jacEnclosure;

   using Taylor<MapType>::getField;
   using Taylor<MapType>::getOrder;
   using Taylor<MapType>::getStep;
   using Taylor<MapType>::setStep;
   using Taylor<MapType>::getCoeff;
   using Taylor<MapType>::getMatrixCoeff;
   using Taylor<MapType>::setParameter;

protected:
   using Taylor<MapType>::computeMatrixCoeff;
   using Taylor<MapType>::m_order;
   using Taylor<MapType>::m_dimension;
   using Taylor<MapType>::m_step;
   using Taylor<MapType>::m_vField;
   using Taylor<MapType>::m_coeff;
   using Taylor<MapType>::m_matrixCoeff;
   using Taylor<MapType>::m_F;

   using BasicC2Taylor<MapType>::m_c2MatrixCoeff;
   using BasicC2Taylor<MapType>::computeC2Coeff;
}; // the end of class C2Taylor

// --------------------------------------------------

template<typename MapType>
inline C2Taylor<MapType>& C2Taylor<MapType>::operator=(const C2Taylor& a_t)
{
   BasicC2Taylor<MapType>::operator=(a_t);
   return *this;
}

}} // the end of the namespace capd::dynsys

#endif // _C2TaylIntf_H

/// @}
