/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file odentaylintf.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _ODENTAYLINTF_H_
#define _ODENTAYLINTF_H_

#include "dynsys/dynsysintf.h"

namespace capd{
namespace dynsys{

template <typename MatrixT>
class OdeNumTaylor: public OdeNum<MatrixT>{
public:
   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   using OdeNum<MatrixType>::ode;
   using OdeNum<MatrixType>::order;
   using OdeNum<MatrixType>::step;

   OdeNumTaylor(Ode<MatrixType> &a_ode, int a_order, const ScalarType &a_step);

   VectorType Phi(const VectorType &iv) const;
   MatrixType JacPhi(const VectorType &iv) const;
   VectorType Remainder(const VectorType &iv) const;

private:
   void tabulateCoefficients(const VectorType &x, int upto, VectorType coeficients[]) const;
};

// inline definitions

template <typename MatrixType>
inline OdeNumTaylor<MatrixType>::OdeNumTaylor(Ode<MatrixType> &a_ode, int a_order,const ScalarType &a_step)
   : OdeNum<MatrixType>(a_ode,a_order,a_step)
{}

}} // namespace capd::dynsys

#endif

/// @}
