/// @addtogroup homologicalAlgebra
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file AsLtHomology.hpp
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include <iostream>
#include <fstream>
#include <sstream>
#include <exception>
#include <stdexcept>
#include <new>
using namespace std;

#include "homologicalAlgebra/embeddingDim.h"
#include "auxil/ofstreamcout.h"
#include "auxil/Stopwatch.h"
#include "auxil/memSize.h"
#include "auxil/CRef.h"

extern ofstreamcout fcout;

#include "vectalg/matrixSlice.h"
#include "matrixAlgorithms/intMatrixAlgorithms.hpp"

using namespace capd;
using namespace matrixAlgorithms;

#include "homologicalAlgebra/homologicalAlgebra.hpp"
#include "homologicalAlgebra/readCubFile.hpp"
#include "homologicalAlgebra/homAlgFunctors.hpp"
#include "homologicalAlgebra/cubSetFunctors.hpp"
#include "homologicalAlgebra/acyclicConfigs.hpp"


void AsLtHom(int embDim, const int dims[], const char* bytes, int* betti,int streaming){
  if(streaming) fcout.turnOn();
  else fcout.turnOff();
  if(3 != embDim){
    std::ostringstream s;
    std::cout  << "This executable is for embedding dimension 3 only and the provided set has dimension " << embDim  << std::endl;
    s << "This executable is for embedding dimension " << embeddingDim << " and the provided set has dimension " << embDim  << std::endl;
    throw std::runtime_error(s.str());
  }
  if(dims[0] % 32) throw std::runtime_error("The 0th dimension must be a multiciplity of 32!");
  readAcyclicConfigs();
  CRef<BCubSet> cubSetCR(new BCubSet(dims,bytes));
  CRef<HomologySignature> homSignCR=CubSetFunctors<BCubSet,BCubCelSet>::HomologyViaAcyclicSubspaceLTD3Shaved(cubSetCR);
  for(int i=0;i<embeddingDim;++i) betti[i]=homSignCR().bettiNumber(i);
}

// Note: only one of the two following methods may be uncommented

// This method kept only for testing purposes for the above version of AsLtHom
/* // -------------- keep it here ------------
void AsLtHom(const char* fileName){
  int d[3];
  const int* dims=d;
  const char* bitmap=0;
  int betti[3];
  readCubFile(fileName,dims,bitmap);
  AsLtHom(embeddingDim,dims,bitmap,betti,false);
  for(int i=0;i<embeddingDim;++i) std::cout << "Betti[" << i << "]=" << betti[i]  << std::endl;
}
*/ // -------------- keep it here ------------

void AsLtHom(const char* fileName){
  CRef<BCubSet> cubSetCR=readCubFile<BCubSet,BCubCelSet>(fileName);
  readAcyclicConfigs();
  Stopwatch swTot;
  CRef<HomologySignature> homSignCR=CubSetFunctors<BCubSet,BCubCelSet>::HomologyViaAcyclicSubspaceLTD3Shaved(cubSetCR);
  fcout << "Computed homology is: \n" << homSignCR()  << std::endl;
  fcout << "Computation completed in " << swTot  << std::endl;
}
/// @}

