/// @addtogroup map
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file c2mapintf.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak,  2000-2005 */

#ifndef _C2MAPINTF_H
#define _C2MAPINTF_H

#include <string>
#include <stdexcept>
#include <sstream>
#include <vector>
#include <new>
#include "map/mapintf.h"
#include "map/c2coeffintf.h"

namespace capd{
namespace map{

// ----------------------------------------------

template<typename MatrixT>
class C2Map : public Map<MatrixT>
{
public:
   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename VectorType::ScalarType ScalarType;
   typedef Function<VectorType> FunctionType;
   typedef C2Coeff<ScalarType> C2CoeffType;

   C2Map(void);
   C2Map(const std::string &, int order=0);
   C2Map(const char *, int order=0);
   C2Map(const C2Map &);
   virtual ~C2Map(){
      delete []c2der;
   }

   C2Map& operator=(const char *);
   C2Map& operator=(const std::string &);
   C2Map& operator=(const C2Map &);

   void setOrder(int);
   void setParameter(const std::string &name, const ScalarType& value);
   C2CoeffType* computeSecondDerivatives(VectorType [], int) const;
   void computeSecondDerivatives(const VectorType &iv, C2CoeffType &result) const;

   using Map<MatrixType>::variational;
   using Map<MatrixType>::variables;
   using Map<MatrixType>::getDimension;

protected:
   using Map<MatrixType>::dim;
   using Map<MatrixType>::fun;
   using Map<MatrixType>::der;
   using Map<MatrixType>::order;

   FunctionType *c2der; // a table of second partial derivaties, c2der = new FunctionType[dim*dim*(dim+1)/2]

private:
   void allocateC2Der();
}; // the end of class Map


}} // the end of the namespace capd::map

#endif //define _C2MAPINTF_H

/// @}
