/// @addtogroup map
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file series.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2006 */

#ifndef _Series_H
#define _Series_H

#include "map/CnContainer.h"
#include "map/node.h"

namespace capd{
namespace map{

template<typename ScalarT>
class SeriesContainer : public CnContainer< Node<ScalarT>* >
{
public:
   typedef ScalarT ScalarType;
   typedef Node<ScalarType> NodeType;
   typedef CnContainer<NodeType*> ContainerType;
   typedef typename ContainerType::iterator iterator;
   typedef typename ContainerType::const_iterator const_iterator;
   typedef typename ContainerType::Multipointer Multipointer;
   typedef typename ContainerType::Multiindex Multiindex;

   using ContainerType::begin;
   using ContainerType::end;

   SeriesContainer(int dim, int rank, int order) : ContainerType(dim,rank)
   {
      iterator b=begin(), e=end();
      while(b!=e)
      {
         (*b) = new ConsNode<ScalarType>(ScalarType(0.),order+1);
         ++((**b).m_links);
         ++b;
      }
   }

   SeriesContainer(int dim, int rank, bool) : ContainerType(dim,rank)
   {
      iterator b=begin(), e=end();
      while(b!=e)
      {
         (*b) = 0;
         ++b;
      }
   }

   ~SeriesContainer()
   {
      iterator b=begin(), e=end();
      while(b!=e)
      {
         if(*b)
            if( !(--(**b).m_links) )
               delete (*b);         
         ++b;
      }
   }

   void setOrder(int a_new)
   {
      iterator b=begin(), e=end();
      while(b!=e)
      {
         if(*b)
            (*b)->setOrder(a_new+1,NULL);
         ++b;
      }
   }
   template<typename VectorType>
   void takeVector(const Multipointer&mp, int r, VectorType& result) const
   {
      typename VectorType::iterator b=result.begin(), e=result.end();
      int ind = this->index(mp);
      int d=0;
      while(b!=e && d<this->dimension())
      {
         const_iterator n = begin(d,mp.dimension()) + ind;
         *b = (*n)->value[r];
         ++b;
         ++d;
      }
   }
};

}} // the end of the namespace capd::map

#endif // _Series_H

/// @}
