/// @addtogroup newton
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file krawczyk.cpp
///
/// @author Tomasz Kapela
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _NEWTON_RESULT_H_
#define _NEWTON_RESULT_H_

namespace capd{
namespace newton{

enum NewtonResult {ResultUndefined = -2,  TooManyIterations = -1, NoZeroes = 0, ZeroExists = 1};

typedef  NewtonResult KrawczykResult;

inline std::string resultToText(NewtonResult code)
{

   switch(code){
        case ZeroExists: return "There exists exactly one zero of a given function in this set.";

        case NoZeroes:  return "There are no zeroes of a given function in this set.";

        case TooManyIterations: return "We cannot conclude. Maximal number of taking intersection was exceeded.";

        case ResultUndefined:  return "\n We cannot conclude. Maybe estimates are no good enough.";

        default: return " Unknown Newton Result code."; 
     }
}

}} // namespace capd::newton

#endif //_NEWTON_RESULT_H_

/// @}
