/// @addtogroup homologicalAlgebra
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file RepSetToCubSet.cpp
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include <iostream>
#include <fstream>
#include <sstream>
#include <exception>
#include <new>
using namespace std;


#include "RepSetToCubSetLib.cpp"
#include "auxil/ofstreamcout.h"
extern ofstreamcout fcout;

void readCubFile(const int*& dims, const char*& bitmap);
void handle_unexpected() {
  fcout << "unexpected exception thrown" << std::endl;
  exit(0);
}

void handle_terminate() {
  fcout << "terminate() called" << std::endl;
  exit(0);
}

typedef unsigned long int word;
static const int embeddingDim=3;

int main(){
  set_unexpected(handle_unexpected);
  set_terminate(handle_terminate);

  fcout.turnOn();
  try{
    RepSet<ElementaryCube> eCubeRepSet;
//std::cout << "Reading\n";
    cin >> eCubeRepSet;
    BmpRepSet<word,embeddingDim> repSet(eCubeRepSet);
//std::cout << "Reading finished\n";
    EuclBitSetT<word,embeddingDim>& cubSet=(EuclBitSetT<word,embeddingDim>&)repSet;
//std::cout << "Writing\n";
    cout << "dimension " << embeddingDim << std::endl;
    cout << "boundingBox ";
    for(int i=0;i<embeddingDim;++i){
      cout << cubSet.getUnpaddedWidth(i) << " ";
    }
    cout << std::endl;
    cout << cubSet;
  }catch(std::exception& e){
    fcout << "Caught exception: " << e.what() << endl;
  }catch(std::string& s){
    fcout << "Caught exception: " << s.c_str() << endl;
  }catch(const char* c){
    fcout << "Caught exception: " << c << endl;
  }catch(...){
    fcout << "Caught an unknown exception: " << endl;
  }
}

/// @}

