/// @addtogroup homology
/// @{

/////////////////////////////////////////////////////////////////////////////
///
/// @file tabulate.cpp
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2007 by Pawel Pilarczyk.
//
// This file is part of the Homology Library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

// Started in January 2002. Last revision: June 2, 2007.


#include "homology/config.h"
#include "homology/textfile.h"
#include "homology/pointset.h"
#include "homology/chains.h"
#include "homology/bitfield.h"
#include "homology/integer.h"
#include "homology/hashsets.h"
#include "homology/gcomplex.h"
#include "homology/cubisets.h"

#include <iostream>
#include <fstream>
#include <iomanip>
#include <ctime>
#include <cstdlib>
#include <cctype>

namespace capd {
namespace homology {


// --------------------------------------------------
// ------------ tabulated neighborhoods -------------
// --------------------------------------------------

char *tabulated1d = "\x06";

char *tabulated2d =
	"\x06\x03\x05\x00\x0f\x0a\x0c\x08\x03\x02\x00\x00\x0f\x0a\x0c\x08"
	"\x05\x00\x04\x00\x0f\x0a\x0c\x08\x00\x00\x00\x00\x0f\x0a\x0c\x08";

Tabulated::Tabulated ()
{
	int three = 1;
	for (int dim = 0; dim < maxdim; ++ dim)
	{
		if (dim == 0)
			size [dim] = 0;
		else if (dim == 1)
			size [dim] = 1;
		else
			size [dim] = 1 << (three - 1 - 3);
		deallocate [dim] = false;
		tables [dim] = 0;
	//	sout << "DEBUG: Dim " << dim << ": " << size [dim] <<
	//		" byte(s), " << (three - 1) << " neighbors.\n";
		three *= 3;
	}
	define (1, tabulated1d);
	define (2, tabulated2d);
	return;
} /* Tabulated::Tabulated */

Tabulated::~Tabulated ()
{
	for (int i = 0; i < maxdim; ++ i)
		if (tables [i] && deallocate [i])
			delete [] (tables [i]);
	return;
} /* Tabulated::Tabulated */

int Tabulated::read (int dim, const char *filename)
{
	if ((dim <= 0) || (dim >= maxdim))
		return -1;
	if (!tables [dim] || !deallocate [dim])
	{
		tables [dim] = new char [size [dim]];
		deallocate [dim] = true;
	}
	std::ifstream f;
	f. open (filename, std::ios::in | std::ios::binary);
	f. read (tables [dim], size [dim]);
	if (!f)
		return -1;
	return 0;
} /* Tabulated::read */

int Tabulated::write (int dim, const char *filename) const
{
	if ((dim <= 0) || (dim >= maxdim) || (!tables [dim]))
		return -1;
	std::ofstream f;
	f. open (filename, std::ios::out | std::ios::binary);
	f. write (tables [dim], size [dim]);
	if (!f)
		return -1;
	return 0;
} /* Tabulated::write */

// --------------------------------------------------

Tabulated tabulated;


} // namespace homology
} // namespace capd

// --------------------------------------------------


/// @}

