/// @addtogroup intvtst
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file intvtst.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include <fstream>
#include <cmath>
#include <string>

// To enable compilation with KRAK uncomment the following line
//#define KRAK
// Note that to  compile with KRAK you will also have to add the krak library to the makefile

#if defined(KRAK)
#  include "krak/krak.h"
#else
# define rootFrame cout
# define waitBt()
# define openGW()
# define closeGW()
#endif

#include "interval/doubleInterval.h"

using namespace std;
using capd::rounding::DoubleRounding;

#if defined(KRAK)
void test_templates(void)
{
 rootFrame.Clear();
 istringstream text("[1,2.5][3.5,4.5]");

 interval i,j;
 rootFrame << "text=\"" << text << "\"\n";
 text >> i >> j;
 rootFrame << "After reading i and j from text:\n";
 rootFrame << "i = " << i << " j= " << j << "\n";
 rootFrame << "text=\"" << text << "\"\n";
 rootFrame << "After inserting \"i=value of i\" to text:\n";
 text << "i=" << i;
 rootFrame << "text=\"" << text << "\"\n";
 rootFrame << "And indeed i=" << i << "\n";
 waitBt();
}
#endif


void testExp(interval z)
{
 rootFrame << "z=" << z << "\n";
 interval expintv=exp(z);
 double dexp=exp(z.rightBound());
 rootFrame << "diff in exp: our intervalexp - standardexp " << (expintv - dexp) << "\n";
}

void testExp()
{
#if defined(KRAK)
 rootFrame.Clear();
#endif
 rootFrame << "Testing exp(interval)\n\n";

 testExp(1.01);
 testExp(0.99);
 testExp(0.5);
// ponizej objawia sie blad, ale tylko przy wylaczonej obsludze niedozwolonej operacji przez
//   procesor INTEL (IM = 0 w bitach maskowania bledow numerycznych

 testExp(12.0);
 waitBt();
}


void test_sin(interval z)
{
 rootFrame << "z=" << z << "\n";
 interval sinintv=sin(z);
 double dsin=sin(z.rightBound());
 rootFrame << "diff in sin: our interval sin  - standard sin " << (sinintv - dsin) << "\n";
}

void test_sin()
{
#if defined(KRAK)
 rootFrame.Clear();
#endif
 rootFrame << "Testing sin(interval)\n\n";
 test_sin(pi);
 test_sin(6.0);
 test_sin(1.0);
 waitBt();
}

void show_rounding_test(void)
{
 string result;
 rootFrame << "\nTesting rounding:\n\n";

 DoubleRounding::roundNearest();
 result=(DoubleRounding::test() == 0 ? "OK" : "BAD");
 rootFrame << "Rounding nearest: " << result << "\n";

 DoubleRounding::roundDown();
 result=(DoubleRounding::test() == 1 ? "OK" : "BAD");
 rootFrame << "Rounding down:    " << result << "\n";

 DoubleRounding::roundUp();
 result=(DoubleRounding::test() == 2 ? "OK" : "BAD");
 rootFrame << "Rounding up:      " << result << "\n";

 DoubleRounding::roundCut();
 result=(DoubleRounding::test() == 3 ? "OK" : "BAD");
 rootFrame << "Rounding cut:     " << result << "\n";
 waitBt();
}

int main(int, char*[])
{
 //init_fpunit();

 #ifdef DEBUGGING
   cerr << "DEBUGGING" << endl;
 #endif

 #ifdef KRAK
  cerr << "KRAK" << endl;
  openGW( 760,580,WHITE,BLACK);
 #endif

// test verifying if rounding mechanism works properly
 show_rounding_test();

#ifdef UP_ROUNDING
 DoubleRounding::roundUp();
#endif

// test for the exp(interval) function - it has problems!!!
 testExp();

// simple test of sin function
 test_sin();

// some other tests
#if defined(KRAK)
  test_templates();
#endif
#ifdef KRAK
  closeGW();
#endif
  return 0;
}

/// @}
