/// @addtogroup setarth
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file rossler.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _rossler_h_
#define _rossler_h_

#include "capd/capdlib.h"

class Rossler : public Ode{
public:
	Rossler(const interval& the_a, const interval &the_b);
	intervalVector f(const intervalVector &iv) const;
	intervalMatrix df(const intervalVector &iv) const;
	intervalMatrix d2f(int i,const intervalVector &iv) const;
      
//    protected:    
	interval a,b;
};

class RosslerEuler : public DynSys{
public:
	RosslerEuler(Rossler &the_rossler, interval the_h);
	intervalVector Phi(const intervalVector &iv) const;
	intervalMatrix JacPhi(const intervalVector &iv) const;
	intervalVector Remainder(const intervalVector &iv) const;
protected:
	Rossler &rossler;
	interval h;
};


//inline definitions

inline Rossler::Rossler(const interval& the_a, const interval& the_b)
	: a(the_a),b(the_b)
{}

inline RosslerEuler::RosslerEuler(Rossler &the_rossler, interval the_h)
	: rossler(the_rossler),h(the_h)
{}

inline intervalVector RosslerEuler::Remainder(const intervalVector &iv) const
{
	intervalVector result;
	result.clear();
	return result;
}

#endif

/// @}
